/**
 * @file main.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Function Main : program starts here
 * Main documentation page is also described in this file.
 * Documentation is written in the doxygen format.
*/
//-------------------------------------------------------------------------------------------//
/**
 * @mainpage pouetChess : A 3D and Open Source Chess game with an embedded AI
 * @section About
 * This program has been developed under Linux Fedora Core 4 with KDE and
 * written whith Kate (very good editor !). 
 *
 * It compiles without warning with gcc 4.0.x and earlier versions.
 *
 * By the way all the files are encoded in <b>UTF8</b>.
 * @section Introduction
 * pouetChess dependencies :
 *
 * - SDL
 * - SDL_image
 * - OpenGL
 *
 * Hardware configuration required:
 *
 * - Processor : 500Mhz
 * - Memory : 128 Mb
 * - Graphic card : GeForce 2 or Radeon 8500 with latest drivers properly installed
 *
*/


#include "scenes.h"
#include "scene_init.h"
#include "scene_main_menu.h"
#include "scene_main_game.h"

GlobalInfos* pGlobalInfos;







int main(int argc, char *argv[])
{
	// The one and only global Info instance is allocated here
	pGlobalInfos = new GlobalInfos;
	if (pGlobalInfos->loadGameConfiguration()==false)
		return 1;
	
	// SDL Initialization
	if (SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		printf("SDL_Init(SDL_INIT_VIDEO) failed : %s\n",SDL_GetError());
		SDL_Quit();
		return 2;
	}
	
	SDL_WM_SetCaption("pouetChess version "GAME_VERSION,C_DATA_ICON_POUETCHESS);
	
	SDL_Surface* game_icon=SDL_LoadBMP(C_DATA_ICON_POUETCHESS);
	
	if (game_icon)
		SDL_WM_SetIcon(game_icon,NULL);
	
	
	
	// Get video informations in order to get the bpp
	const SDL_VideoInfo* info = NULL;
	info = SDL_GetVideoInfo();

	// Should never happen
	if (info == NULL) 
	{
		printf("SDL_GetVideoInfo() failed: %s\n",SDL_GetError());
		SDL_Quit();
		return 3;
	}
	
	
	// Window creation
	if (pGlobalInfos->est_en_plein_ecran())
	{
		// we want fullscreen mode, if we can't have this mode, try in windowed mode
		if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE | SDL_FULLSCREEN))
		{
			printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
			printf("I try the window mode\n");
			if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE))
			{
				printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
				SDL_Quit();
				return 4;
			}
			else
				printf("%dx%d in window mode is OK\n",pGlobalInfos->largeurEcran(),pGlobalInfos->hauteurEcran());
		}
	}
	else
	{
		if (NULL==SDL_SetVideoMode(pGlobalInfos->largeurEcran(), pGlobalInfos->hauteurEcran(), info->vfmt->BitsPerPixel, SDL_OPENGL | SDL_HWSURFACE))
		{
			printf("SDL_SetVideoMode failed : %s\n", SDL_GetError());
			SDL_Quit();
			return 4;
		}
	}
	
	// OpenGL basic initialization
	glClearColor(0.0f,0.0f,0.0f,1.0f);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	glViewport (0, 0, (GLsizei)(pGlobalInfos->largeurEcran()), (GLsizei)(pGlobalInfos->hauteurEcran()));
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (45.0f, (GLfloat)(pGlobalInfos->largeurEcran())/(GLfloat)(pGlobalInfos->hauteurEcran()),
					1.0f, 10000.0f);		
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	
	// clear screen
	SDL_GL_SwapBuffers();
	SDL_Delay(25);
	
	// Allocate scene (we do not load anything here)
	Echec_Scene* echec_scene[NOMBRE_DE_SCENES];
	
	echec_scene[0] = new E_scene_init;
	echec_scene[1] = new E_scene_intro;
	echec_scene[2] = new E_scene_menu_principal;
	echec_scene[3] = new E_scene_jeu_principal;
	
	// first scene is SCENE_INIT.
	pGlobalInfos->changer_de_scene(E_SCENE_INIT);

	// main game loop
	while( pGlobalInfos->le_jeu_est_actif() )
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear screen
		glLoadIdentity();											// Load Identity matrix
		
		pGlobalInfos->actualiser();									// update game events and other things
		echec_scene[pGlobalInfos->sceneActuelle()]->actualiser();	// update the game
		
		SDL_GL_SwapBuffers();										// swap buffers : the scene is now displayed
	}
	
	// Game is done : unload everything
	for(int i=0;i<NOMBRE_DE_SCENES;i++)
	{
		echec_scene[i]->decharger();
		delete echec_scene[i];
	}
			
	delete pGlobalInfos;
	pGlobalInfos=NULL;
	
	SDL_Quit();
	return(0);
}

